@@ -1,5 +1,9 @@
+import dataclasses
 import json
 import time
+from dataclasses import dataclass
+from enum import Enum
+from typing import List
 from urllib.parse import urlencode, urljoin
 from uuid import UUID
 
@@ -48,9 +52,35 @@ class InvalidStateError(WaldurClientException):
     pass
 
 
+@dataclass
+class ComponentUsage:
+    # TODO: rename to 'component_type' after https://opennode.atlassian.net/browse/WAL-4259 is done
+    type: str
+    amount: int
+    description: str = ''
+
+
+@dataclass
+class ResourceReportRecord:
+    header: str
+    body: str
+
+
+class ResourceState(Enum):
+    OK = 'ok'
+    ERRED = 'erred'
+    TERMINATED = 'terminated'
+
+
+class ProjectRole(Enum):
+    ADMINISTRATOR = 'admin'
+    MANAGER = 'manager'
+    MEMBER = 'member'
+
+
 class WaldurClient(object):
     class Endpoints(object):
-        Provider = 'openstacktenant'
+        Provider = 'service-settings'
         Project = 'projects'
         Flavor = 'openstacktenant-flavors'
         Subnet = 'openstacktenant-subnets'
@@ -64,13 +94,22 @@ class WaldurClient(object):
         TenantSecurityGroup = 'openstack-security-groups'
         Volume = 'openstacktenant-volumes'
         VolumeType = 'openstacktenant-volume-types'
-        OpenStackPackage = 'openstack-packages'
         MarketplaceOffering = 'marketplace-offerings'
         MarketplacePlan = 'marketplace-plans'
         MarketplaceOrder = 'marketplace-orders'
+        MarketplaceOrderItem = 'marketplace-order-items'
         MarketplaceResources = 'marketplace-resources'
         MarketplaceCategories = 'marketplace-categories'
         Customers = 'customers'
+        Invoice = 'invoices'
+        ComponentUsage = 'marketplace-component-usages'
+        RemoteEduteams = 'remote-eduteams'
+        ProjectPermissions = 'project-permissions'
+        CustomerPermissions = 'customer-permissions'
+        OfferingPermissions = 'marketplace-offering-permissions'
+        Users = 'users'
+        UserInvitations = 'user-invitations'
+        OfferingUsers = 'marketplace-offering-users'
 
     marketplaceScopeEndpoints = {
         'OpenStackTenant.Instance': Endpoints.Instance,
@@ -96,8 +135,8 @@ class WaldurClient(object):
     def _build_url(self, endpoint):
         return urljoin(self.api_url, self._ensure_trailing_slash(endpoint))
 
-    def _build_resource_url(self, endpoint, uuid, action=None):
-        parts = [endpoint, uuid]
+    def _build_resource_url(self, endpoint, uid, action=None):
+        parts = [endpoint, str(uid)]
         if action:
             parts.append(action)
         return self._build_url('/'.join(parts))
@@ -134,12 +173,13 @@ class WaldurClient(object):
             error = self._parse_error(response)
             raise WaldurClientException(error)
 
-        return response.json()
+        if response.text:
+            return response.json()
+        return ''
 
-    def _get_paginated_data(self, url, **kwargs):
+    def _get_all(self, url, **kwargs):
         params = dict(headers=self.headers)
         params.update(kwargs)
-        params['page_size'] = 200
 
         try:
             response = requests.get(url, **params)
@@ -150,6 +190,8 @@ class WaldurClient(object):
             error = self._parse_error(response)
             raise WaldurClientException(error)
         result = response.json()
+        if 'Link' not in response.headers:
+            return result
         while 'next' in response.headers['Link']:
             if 'prev' in response.headers['Link']:
                 next_url = response.headers['Link'].split(', ')[2].split('; ')[0][1:-1]
@@ -235,7 +277,10 @@ class WaldurClient(object):
 
     def _query_resource_list(self, endpoint, query_params):
         url = self._build_url(endpoint)
-        return self._get_paginated_data(url, params=query_params)
+        if query_params is None:
+            query_params = {}
+        query_params.setdefault('page_size', 200)
+        return self._get_all(url, params=query_params)
 
     def _get_resource(self, endpoint, value, extra=None):
         """
@@ -270,12 +315,23 @@ class WaldurClient(object):
         url = self._build_resource_url(endpoint, uuid, action)
         return self._post(url, [202], **kwargs)
 
-    def _get_provider(self, identifier):
+    def _get_service_settings(self, identifier):
         return self._get_resource(self.Endpoints.Provider, identifier)
 
     def _get_project(self, identifier):
         return self._get_resource(self.Endpoints.Project, identifier)
 
+    def get_user(self, identifier):
+        return self._get_resource(self.Endpoints.Users, identifier)
+
+    def list_users(self):
+        url = self._build_url(self.Endpoints.Users)
+        return self._get_all(url)
+
+    def list_ssh_keys(self):
+        url = self._build_url(self.Endpoints.SshKey)
+        return self._get_all(url)
+
     def _get_property(self, endpoint, identifier, settings_uuid):
         query = {'settings_uuid': settings_uuid}
         if is_uuid(identifier):
@@ -517,106 +573,6 @@ class WaldurClient(object):
 
         return response
 
-    def create_instance(
-        self,
-        name,
-        provider,
-        project,
-        networks,
-        image,
-        system_volume_size,
-        description=None,
-        flavor=None,
-        flavor_min_cpu=None,
-        flavor_min_ram=None,
-        interval=10,
-        timeout=600,
-        wait=True,
-        ssh_key=None,
-        data_volume_size=None,
-        security_groups=None,
-        tags=None,
-        user_data=None,
-        check_mode=False,
-    ):
-        """
-        Creates OpenStack instance from passed parameters.
-
-        :param name: name of the instance.
-        :param description: description of the instance.
-        :param provider: uuid or name of the provider to use.
-        :param project: uuid or name of the project to add the instance.
-        :param networks: a list of networks to attach instance to.
-        :param flavor: uuid or name of the flavor to use.
-        :param flavor_min_cpu: min cpu count.
-        :param flavor_min_ram: min ram size (MB).
-        :param image: uuid or name of the image to use.
-        :param system_volume_size: size of the system volume in GB.
-        :param interval: interval of instance state polling in seconds.
-        :param timeout: a maximum amount of time to wait for instance provisioning.
-        :param wait: defines whether the client has to wait for instance provisioning.
-        :param ssh_key: uuid or name of the ssh key to add to the instance.
-        :param data_volume_size: size of the data volume in GB.
-            No data volume is going to be created if empty.
-        :param security_groups: list of security groups to add to the instance.
-        :param tags: list of tags to add to the instance.
-        :param user_data: additional data that will be added to the instance.
-        :return: an instance as a dictionary.
-        """
-        provider = self._get_provider(provider)
-        settings_uuid = provider['settings_uuid']
-        project = self._get_project(project)
-        if flavor:
-            flavor = self._get_flavor(flavor, settings_uuid)
-        else:
-            flavor = self._get_flavor_from_params(flavor_min_cpu, flavor_min_ram)
-
-        image = self._get_image(image, settings_uuid)
-        subnets, floating_ips = self._networks_to_payload(networks)
-
-        payload = {
-            'name': name,
-            'flavor': flavor['url'],
-            'image': image['url'],
-            'service_settings': provider['settings'],
-            'project': project['url'],
-            'system_volume_size': system_volume_size * 1024,
-            'internal_ips_set': subnets,
-            'floating_ips': floating_ips,
-        }
-
-        if security_groups:
-            payload['security_groups'] = []
-            for group in security_groups:
-                security_group = self._get_security_group(group, settings_uuid)
-                payload['security_groups'].append({'url': security_group['url']})
-
-        if data_volume_size:
-            payload.update({'data_volume_size': data_volume_size * 1024})
-        if user_data:
-            payload.update({'user_data': user_data})
-        if ssh_key:
-            ssh_key = self._get_resource(self.Endpoints.SshKey, ssh_key)
-            payload.update({'ssh_public_key': ssh_key['url']})
-        if tags:
-            payload.update({'tags': tags})
-        if description:
-            payload['description'] = description
-
-        if check_mode:
-            return payload
-
-        instance = self._create_instance(payload)
-
-        if wait:
-            self._wait_for_resource(
-                self.Endpoints.Instance, instance['uuid'], interval, timeout
-            )
-            if floating_ips:
-                self._wait_for_external_ip(instance['uuid'], interval, timeout)
-
-        return instance
-
     def _get_project_resource(self, endpoint, name, project=None):
         if is_uuid(name):
             return self._query_resource_by_uuid(endpoint, name)
@@ -636,9 +592,9 @@ class WaldurClient(object):
     def get_marketplace_resource_scope(self, name, offering_type, project=None):
         """Get marketplace resource scope. Depending on the offering type scope type can be different.
 
-            :param name: name of the scope.
-            :param offering_type: marketplace offering type.
-            :param project: project UUID or name.
+        :param name: name of the scope.
+        :param offering_type: marketplace offering type.
+        :param project: project UUID or name.
         """
 
         if not is_uuid(name) and not project:
@@ -681,11 +637,74 @@ class WaldurClient(object):
 
         return result[0], scope
 
+    def get_marketplace_resource(self, resource_uuid):
+        return self._get_resource(
+            WaldurClient.Endpoints.MarketplaceResources, resource_uuid
+        )
+
+    def list_marketplace_resources(
+        self,
+        provider_uuid: str = None,
+        state: str = None,
+        offering_uuid: str = None,
+        fields: List[str] = None,
+    ):
+        params = {}
+        if provider_uuid is not None:
+            params['provider_uuid'] = provider_uuid
+        if state is not None:
+            params['state'] = state
+        if offering_uuid is not None:
+            params['offering_uuid'] = offering_uuid
+        if fields is not None:
+            if type(fields) is not list:
+                fields = [fields]
+            params['field'] = fields
+
+        return self._query_resource_list(self.Endpoints.MarketplaceResources, params,)
+
+    def marketplace_resource_set_state(self, resource_uuid: str, state: ResourceState):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources,
+            resource_uuid,
+            action='set_state_by_provider',
+        )
+        payload = {'state': state.value}
+        return self._post(url, valid_states=[200], json=payload)
+
+    def marketplace_resource_set_backend_id(self, resource_uuid: str, backend_id: str):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources, resource_uuid, action='set_backend_id',
+        )
+        payload = {'backend_id': backend_id}
+        return self._post(url, valid_states=[200], json=payload)
+
+    def marketplace_resource_submit_report(
+        self, resource_uuid: str, report: List[ResourceReportRecord]
+    ):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources, resource_uuid, action='submit_report'
+        )
+        payload = {'report': [dataclasses.asdict(record) for record in report]}
+        return self._post(url, valid_states=[200], json=payload)
+
+    def marketplace_resource_get_team(self, resource_uuid: str):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources, resource_uuid, action='team'
+        )
+        return self._get(url, valid_states=[200])
+
+    def marketplace_resource_get_plan_periods(self, resource_uuid: str):
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources, resource_uuid, action='plan_periods'
+        )
+        return self._get(url, valid_states=[200])
+
     def get_instance_via_marketplace(self, name, project=None):
         """Get an openstack instance via marketplace.
 
-            :param name: name of the instance.
-            :param project: project UUID or name.
+        :param name: name of the instance.
+        :param project: project UUID or name.
         """
         resource, instance = self.get_marketplace_resource_scope(
             name, 'OpenStackTenant.Instance', project
@@ -695,8 +714,8 @@ class WaldurClient(object):
     def get_volume_via_marketplace(self, name, project=None):
         """Get an openstack volume via marketplace.
 
-            :param name: name of the volume.
-            :param project: project UUID or name.
+        :param name: name of the volume.
+        :param project: project UUID or name.
         """
         resource, instance = self.get_marketplace_resource_scope(
             name, 'OpenStackTenant.Volume', project
@@ -776,55 +795,6 @@ class WaldurClient(object):
     def delete_volume(self, uuid):
         return self._delete_resource(self.Endpoints.Volume, uuid)
 
-    def create_volume(
-        self,
-        name,
-        project,
-        provider,
-        size,
-        description=None,
-        tags=None,
-        wait=True,
-        interval=10,
-        timeout=600,
-    ):
-        """
-        Creates OpenStack volume via Waldur API from passed parameters.
-
-        :param name: name of the volume.
-        :param project: uuid or name of the project to add the volume to.
-        :param provider: uuid or name of the provider to use.
-        :param size: size of the volume in GBs.
-        :param description: arbitrary text.
-        :param tags: list of tags to add to the volume.
-        :param wait: defines whether the client has to wait for volume provisioning.
-        :param interval: interval of volume state polling in seconds.
-        :param timeout: a maximum amount of time to wait for volume provisioning.
-        :return: volume as a dictionary.
-        """
-        provider = self._get_provider(provider)
-        project = self._get_project(project)
-
-        payload = {
-            'name': name,
-            'service_settings': provider['settings'],
-            'project': project['url'],
-            'size': size * 1024,
-        }
-        if description:
-            payload.update({'description': description})
-        if tags:
-            payload.update({'tags': tags})
-
-        resource = self._create_resource(self.Endpoints.Volume, payload)
-
-        if wait:
-            self._wait_for_resource(
-                self.Endpoints.Volume, resource['uuid'], interval, timeout
-            )
-
-        return resource
-
     def detach_volume(self, uuid, wait=True, interval=10, timeout=600):
         """
         Detach OpenStack volume from instance and wait until operation is completed.
@@ -981,33 +951,49 @@ class WaldurClient(object):
         :param attributes: order item attributes.
         :param limits: order item limits.
         """
-        project_url = self._get_project(project)['url']
-        offering_url = self._get_offering(offering, project)['url']
+        project_uuid = self._get_project(project)['uuid']
+        offering_uuid = self._get_offering(offering, project)['uuid']
+        plan_uuid = plan and self._get_plan(plan)['uuid']
+        return self.marketplace_resource_create_order(
+            project_uuid, offering_uuid, plan_uuid, attributes, limits
+        )
 
-        attributes = attributes or {}
-        limits = limits or {}
-        order_item = {
-            'offering': offering_url,
-            'attributes': attributes,
-            'limits': limits,
-        }
+    def get_order(self, order_uuid):
+        return self._get_resource(WaldurClient.Endpoints.MarketplaceOrder, order_uuid)
 
-        if plan:
-            plan_url = self._get_plan(plan)['url']
-            order_item['plan'] = plan_url
+    def list_orders(self, filters=None):
+        return self._query_resource_list(self.Endpoints.MarketplaceOrder, filters)
 
-        # TODO: replace with checkbox data from frontend
-        order_item['accepting_terms_of_service'] = True
+    def get_order_item(self, order_item_uuid):
+        return self._get_resource(
+            WaldurClient.Endpoints.MarketplaceOrderItem, order_item_uuid
+        )
 
-        payload = {
-            'project': project_url,
-            'items': [order_item],
-        }
-        return self._create_resource(self.Endpoints.MarketplaceOrder, payload=payload)
+    def _get_resource_from_creation_order(
+        self, order_uuid, resource_field='resource_uuid', interval=10, timeout=600,
+    ):
+        waited = 0
+        while True:
+            order = self.get_order(order_uuid)
+            if order['items'][0]['state'] == 'erred':
+                raise InvalidStateError(order['items'][0]['error_message'])
+
+            resource_uuid = order['items'][0].get(resource_field)
+            if resource_uuid:
+                return resource_uuid
+            time.sleep(interval)
+
+            waited += interval
+            if waited >= timeout:
+                error = (
+                    'Resource reference has not been found from order item "%s" '
+                    % order_uuid
+                )
+                message = '%s. Seconds passed: %s' % (error, timeout)
+                raise TimeoutError(message)
 
     def _create_scope_via_marketplace(
         self,
-        name,
         offering,
         project,
         attributes,
@@ -1020,7 +1006,6 @@ class WaldurClient(object):
         """
         Create marketplace resource scope via marketplace.
 
-        :param name: the name of scope.
         :param offering: the name or UUID of marketplace offering.
         :param project: the name or UUID of the project.
         :param attributes: order item attributes.
@@ -1029,10 +1014,9 @@ class WaldurClient(object):
         :param timeout: a maximum amount of time to wait for instance provisioning.
         :param wait: defines whether the client has to wait for instance provisioning.
         :param check_mode: True for check mode.
-        :return: scope.
+        :return: resource_uuid.
         """
         offering = self._get_offering(offering, project)
-        offering_type = offering['type']
 
         if check_mode:
             return {
@@ -1045,34 +1029,28 @@ class WaldurClient(object):
             project, offering['uuid'], attributes=attributes
         )
         order_uuid = order['uuid']
-        scope = None
-        waited = 0
-        while not scope:
-            time.sleep(interval)
-            order = self._get_resource(
-                WaldurClient.Endpoints.MarketplaceOrder, order_uuid
-            )
-            if order['items'][0]['state'] == 'erred':
-                raise InvalidStateError(order['items'][0]['error_message'])
 
-            try:
-                resource, scope = self.get_marketplace_resource_scope(
-                    name, offering_type, project
-                )
-            except ObjectDoesNotExist:
-                pass
-            waited += interval
-            if waited >= timeout:
-                error = (
-                    'Marketplace resource of scope with name "%s" is not found.' % name
-                )
-                message = '%s. Seconds passed: %s' % (error, timeout)
-                raise TimeoutError(message)
+        resource_uuid = self._get_resource_from_creation_order(order_uuid)
 
         if wait:
-            self._wait_for_resource(scope_endpoint, scope['uuid'], interval, timeout)
+            self._wait_for_resource(scope_endpoint, resource_uuid, interval, timeout)
 
-        return scope
+        return resource_uuid
+
+    def create_resource_via_marketplace(
+        self, project_uuid, offering_uuid, plan_uuid, attributes, limits
+    ):
+        order = self.create_marketplace_order(
+            project_uuid, offering_uuid, plan_uuid, attributes, limits
+        )
+        order_uuid = order['uuid']
+        marketplace_resource_uuid = self._get_resource_from_creation_order(
+            order_uuid, 'marketplace_resource_uuid'
+        )
+        return {
+            'create_order_uuid': order_uuid,
+            'marketplace_resource_uuid': marketplace_resource_uuid,
+        }
 
     def create_instance_via_marketplace(
         self,
@@ -1103,7 +1081,7 @@ class WaldurClient(object):
 
         :param name: name of the instance.
         :param description: description of the instance.
-        :param provider: uuid or name of the provider to use.
+        :param offering: the name or UUID of the offering
         :param project: uuid or name of the project to add the instance.
         :param networks: a list of networks to attach instance to.
         :param flavor: uuid or name of the flavor to use.
@@ -1169,8 +1147,7 @@ class WaldurClient(object):
             volume_type = self._get_volume_type(data_volume_type, settings_uuid)
             attributes.update({'data_volume_type': volume_type['url']})
 
-        instance = self._create_scope_via_marketplace(
-            name,
+        resource_uuid = self._create_scope_via_marketplace(
             offering['uuid'],
             project,
             attributes,
@@ -1182,19 +1159,13 @@ class WaldurClient(object):
         )
 
         if wait and floating_ips:
-            self._wait_for_external_ip(instance['uuid'], interval, timeout)
+            self._wait_for_external_ip(resource_uuid, interval, timeout)
 
-        return instance
+        return resource_uuid
 
     def _delete_scope_via_marketplace(self, scope_uuid, offering_type, options=None):
-        if options:
-            options = {'attributes': options}
         resource, scope = self.get_marketplace_resource_scope(scope_uuid, offering_type)
-        url = self._build_resource_url(
-            self.Endpoints.MarketplaceResources, resource['uuid'], action='terminate'
-        )
-        order_uuid = self._post(url, valid_states=[200], json=options)['order_uuid']
-        return order_uuid
+        return self.marketplace_resource_terminate_order(resource['uuid'], options)
 
     def delete_instance_via_marketplace(self, instance_uuid, **kwargs):
         """
@@ -1224,9 +1195,9 @@ class WaldurClient(object):
 
         :param name: name of the volume.
         :param project: uuid or name of the project to add the volume to.
-        :param provider: uuid or name of the provider to use.
+        :param offering: the name or UUID of the offering
         :param size: size of the volume in GBs.
-        :param type: uuid or name of volume type.
+        :param volume_type: uuid or name of volume type.
         :param description: arbitrary text.
         :param tags: list of tags to add to the volume.
         :param wait: defines whether the client has to wait for volume provisioning.
@@ -1252,7 +1223,6 @@ class WaldurClient(object):
             attributes.update({'type': volume_type['url']})
 
         return self._create_scope_via_marketplace(
-            name,
             offering['uuid'],
             project,
             attributes,
@@ -1298,6 +1268,306 @@ class WaldurClient(object):
 
             return resource, True
 
+    def list_customers(self, filters=None):
+        return self._query_resource_list(self.Endpoints.Customers, filters)
+
+    def create_customer(
+        self,
+        name,
+        email="",
+        address="",
+        registration_code="",
+        backend_id="",
+        abbreviation="",
+        bank_account="",
+        bank_name="",
+        contact_details="",
+        country="",
+        display_name="",
+        domain="",
+        homepage="",
+        native_name="",
+        latitude=None,
+        longitude=None,
+        phone_number="",
+        postal="",
+        vat_code="",
+    ):
+        payload = {
+            'abbreviation': abbreviation,
+            'address': address,
+            'bank_account': bank_account,
+            'bank_name': bank_name,
+            'contact_details': contact_details,
+            'country': country,
+            'display_name': display_name,
+            'domain': domain,
+            'email': email,
+            'homepage': homepage,
+            'name': name,
+            'native_name': native_name,
+            'registration_code': registration_code,
+            'backend_id': backend_id,
+            'latitude': latitude,
+            'longitude': longitude,
+            'phone_number': phone_number,
+            'postal': postal,
+            'vat_code': vat_code,
+        }
+        return self._create_resource(self.Endpoints.Customers, payload=payload)
+
+    def delete_customer(self, customer):
+        """
+        Delete a customer by UUID or URL
+
+        :param customer: customer's UUID or URL
+        :return: deleted customer information
+        """
+        if is_uuid(customer):
+            return self._delete_resource(self.Endpoints.Customers, customer)
+        return self._delete_resource_by_url(customer)
+
+    def list_projects(self, filters=None):
+        return self._query_resource_list(self.Endpoints.Project, filters)
+
+    def create_project(self, customer_uuid, name, backend_id=None):
+        payload = {
+            'name': name,
+            'customer': self._build_resource_url(
+                self.Endpoints.Customers, customer_uuid
+            ),
+            'backend_id': backend_id,
+        }
+        return self._create_resource(self.Endpoints.Project, payload=payload)
+
+    def delete_project(self, project):
+        """
+        Delete a project by UUID or URL
+
+        :param project: project's UUID or URL
+        :return: deleted project information
+        """
+        if is_uuid(project):
+            return self._delete_resource(self.Endpoints.Project, project)
+        return self._delete_resource_by_url(project)
+
+    def list_marketplace_offerings(self, filters=None):
+        return self._query_resource_list(self.Endpoints.MarketplaceOffering, filters)
+
+    def get_marketplace_offering(self, offering_uuid):
+        return self._query_resource_by_uuid(
+            self.Endpoints.MarketplaceOffering, offering_uuid
+        )
+
+    def marketplace_resource_create_order(
+        self, project_uuid, offering_uuid, plan_uuid=None, attributes=None, limits=None
+    ):
+        attributes = attributes or {}
+        limits = limits or {}
+        order_item = {
+            'offering': self._build_resource_url(
+                self.Endpoints.MarketplaceOffering, offering_uuid
+            ),
+            'attributes': attributes,
+            'limits': limits,
+        }
+
+        if plan_uuid:
+            order_item['plan'] = self._build_resource_url(
+                self.Endpoints.MarketplacePlan, plan_uuid
+            )
+
+        # TODO: replace with checkbox data from frontend
+        order_item['accepting_terms_of_service'] = True
+
+        payload = {
+            'project': self._build_resource_url(self.Endpoints.Project, project_uuid),
+            'items': [order_item],
+        }
+        return self._create_resource(self.Endpoints.MarketplaceOrder, payload=payload)
+
+    def marketplace_resource_update_limits_order(self, resource_uuid, limits):
+        payload = {'limits': limits}
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources, resource_uuid, action='update_limits'
+        )
+        return self._post(url, valid_states=[200], json=payload)['order_uuid']
+
+    def marketplace_resource_terminate_order(self, resource_uuid, options=None):
+        if options:
+            options = {'attributes': options}
+        url = self._build_resource_url(
+            self.Endpoints.MarketplaceResources, resource_uuid, action='terminate'
+        )
+        return self._post(url, valid_states=[200], json=options)['order_uuid']
+
+    def get_invoice_for_customer(self, customer_uuid, year, month):
+        return self._query_resource(
+            self.Endpoints.Invoice,
+            {'customer_uuid': customer_uuid, 'year': year, 'month': month},
+        )
+
+    def list_component_usages(self, resource_uuid, date_after=None, date_before=None):
+        return self._query_resource_list(
+            self.Endpoints.ComponentUsage,
+            {
+                'resource_uuid': resource_uuid,
+                'date_after': date_after,
+                'date_before': date_before,
+            },
+        )
+
+    def create_component_usages(
+        self, plan_period_uuid: str, usages: List[ComponentUsage]
+    ):
+        url = self._build_url(f'{self.Endpoints.ComponentUsage}/set_usage/')
+        payload = {
+            'plan_period': plan_period_uuid,
+            'usages': [dataclasses.asdict(usage) for usage in usages],
+        }
+        return self._post(url, valid_states=[201], json=payload)
+
+    def get_remote_eduteams_user(self, cuid):
+        return self._create_resource(
+            self.Endpoints.RemoteEduteams, {'cuid': cuid,}, valid_state=200,
+        )
+
+    def create_project_permission(
+        self, user_uuid, project_uuid, role, expiration_time=None
+    ):
+        return self._create_resource(
+            self.Endpoints.ProjectPermissions,
+            {
+                'user': self._build_resource_url(self.Endpoints.Users, user_uuid),
+                'project': self._build_resource_url(
+                    self.Endpoints.Project, project_uuid
+                ),
+                'role': role,
+                'expiration_time': expiration_time,
+            },
+        )
+
+    def get_project_permissions(self, project_uuid, user_uuid=None, role=None):
+        query_params = {
+            'project': project_uuid,
+        }
+        if role:
+            query_params['role'] = role
+        if user_uuid:
+            query_params['user'] = user_uuid
+
+        return self._query_resource_list(
+            self.Endpoints.ProjectPermissions, query_params
+        )
+
+    def update_project_permission(self, permission_id, new_expiration_time):
+        return self._update_resource(
+            self.Endpoints.ProjectPermissions,
+            permission_id,
+            {'expiration_time': new_expiration_time},
+        )
+
+    def remove_project_permission(self, permission_id):
+        return self._delete_resource(self.Endpoints.ProjectPermissions, permission_id)
+
+    def create_customer_permission(
+        self, user_uuid, customer_uuid, role, expiration_time=None
+    ):
+        return self._create_resource(
+            self.Endpoints.CustomerPermissions,
+            {
+                'user': self._build_resource_url(self.Endpoints.Users, user_uuid),
+                'customer': self._build_resource_url(
+                    self.Endpoints.Customers, customer_uuid
+                ),
+                'role': role,
+                'expiration_time': expiration_time,
+            },
+        )
+
+    def get_customer_permissions(self, customer_uuid, user_uuid=None, role=None):
+        query_params = {
+            'customer': customer_uuid,
+        }
+        if role:
+            query_params['role'] = role
+        if user_uuid:
+            query_params['user'] = (user_uuid,)
+
+        return self._query_resource_list(
+            self.Endpoints.CustomerPermissions, query_params
+        )
+
+    def update_customer_permission(self, permission_id, new_expiration_time):
+        return self._update_resource(
+            self.Endpoints.CustomerPermissions,
+            permission_id,
+            {'expiration_time': new_expiration_time},
+        )
+
+    def remove_customer_permission(self, permission_id):
+        return self._delete_resource(self.Endpoints.CustomerPermissions, permission_id)
+
+    def create_offering_permission(self, user_uuid, offering_uuid):
+        return self._create_resource(
+            self.Endpoints.OfferingPermissions,
+            {
+                'user': self._build_resource_url(self.Endpoints.Users, user_uuid),
+                'offering': self._build_resource_url(
+                    self.Endpoints.MarketplaceOffering, offering_uuid
+                ),
+            },
+        )
+
+    def get_offering_permissions(self, offering_uuid, user_uuid=None):
+        query_params = {
+            'offering_uuid': offering_uuid,
+        }
+        if user_uuid:
+            query_params['user'] = user_uuid
+
+        return self._query_resource_list(
+            self.Endpoints.OfferingPermissions, query_params,
+        )
+
+    def remove_offering_permission(self, permission_id):
+        return self._delete_resource(self.Endpoints.OfferingPermissions, permission_id)
+
+    def create_project_invitation(
+        self, email: str, project: str, project_role: ProjectRole
+    ):
+        if is_uuid(project):
+            project = self._build_resource_url(self.Endpoints.Project, project)
+
+        payload = {
+            'email': email,
+            'project': project,
+            'project_role': project_role.value,
+        }
+
+        return self._create_resource(self.Endpoints.UserInvitations, payload)
+
+    def create_remote_offering_user(
+        self, offering: str, user: str, username: str = None
+    ):
+        if is_uuid(offering):
+            offering = self._build_resource_url(
+                self.Endpoints.MarketplaceOffering, offering
+            )
+
+        if is_uuid(user):
+            user = self._build_resource_url(self.Endpoints.Users, user)
+
+        payload = {
+            'offering': offering,
+            'user': user,
+        }
+
+        if username is not None:
+            payload['username'] = username
+
+        return self._create_resource(self.Endpoints.OfferingUsers, payload)
+
 
 def waldur_full_argument_spec(**kwargs):
     spec = dict(
